import ipaddress
from uuid import UUID

from orchestrator import workflow
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle
from orchestrator.workflow import StepList, done, init, step
from orchestrator.workflows.steps import resync, set_status, store_process_subscription

from gso.products.product_blocks import router as router_pb
from gso.products.product_blocks.router import RouterRole, RouterVendor
from gso.products.product_types import router
from gso.products.product_types.router import RouterInactive
from gso.products.product_types.site import Site
from utils.types.ip_port import PortNumber
from gso.products import ProductType
from gso.services import subscriptions
from gso.services.crm import get_customer_by_name


def _get_site_by_name(site_name: str) -> Site:
    """Get a site by its name.

    Args:
    ----
    site_name (str): The name of the site.
    """
    subscription = subscriptions.get_active_site_subscription_by_name(site_name)
    if not subscription:
        raise ValueError(f"Site with name {site_name} not found.")

    return Site.from_subscription(subscription.subscription_id)


@step("Create subscription")
def create_subscription(customer: str) -> State:
    customer_id = get_customer_by_name(customer)["id"]
    product_id: UUID = subscriptions.get_product_id_by_name(ProductType.ROUTER)
    subscription = RouterInactive.from_product_id(product_id, customer_id)

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


def initial_input_form_generator() -> FormGenerator:
    class ImportRouter(FormPage):
        class Config:
            title = "Import Router"

        customer: str
        router_site: str
        hostname: str
        ts_port: int
        router_vendor: RouterVendor
        router_role: RouterRole
        is_ias_connected: bool
        router_lo_ipv4_address: ipaddress.IPv4Address
        router_lo_ipv6_address: ipaddress.IPv6Address
        router_lo_iso_address: str
        router_si_ipv4_network: ipaddress.IPv4Network | None = None
        router_ias_lt_ipv4_network: ipaddress.IPv4Network | None = None
        router_ias_lt_ipv6_network: ipaddress.IPv6Network | None = None

    user_input = yield ImportRouter

    return user_input.dict()


@step("Initialize subscription")
def initialize_subscription(
    subscription: RouterInactive,
    hostname: str,
    ts_port: PortNumber,
    router_vendor: router_pb.RouterVendor,
    router_site: str,
    router_role: router_pb.RouterRole,
    is_ias_connected: bool | None = None,
    router_lo_ipv4_address: ipaddress.IPv4Address | None = None,
    router_lo_ipv6_address: ipaddress.IPv6Address | None = None,
    router_lo_iso_address: str | None = None,
    router_si_ipv4_network: ipaddress.IPv4Network | None = None,
    router_ias_lt_ipv4_network: ipaddress.IPv4Network | None = None,
    router_ias_lt_ipv6_network: ipaddress.IPv6Network | None = None,
) -> State:
    subscription.router.router_ts_port = ts_port
    subscription.router.router_vendor = router_vendor
    subscription.router.router_site = _get_site_by_name(router_site).site
    fqdn = (
        f"{hostname}.{subscription.router.router_site.site_name.lower()}."
        f"{subscription.router.router_site.site_country_code.lower()}"
        ".geant.net"
    )
    subscription.router.router_fqdn = fqdn
    subscription.router.router_role = router_role
    subscription.router.router_access_via_ts = True
    subscription.description = f"Router {fqdn}"
    subscription.router.router_is_ias_connected = is_ias_connected
    subscription.router.router_lo_ipv4_address = router_lo_ipv4_address
    subscription.router.router_lo_ipv6_address = router_lo_ipv6_address
    subscription.router.router_lo_iso_address = router_lo_iso_address
    subscription.router.router_si_ipv4_network = router_si_ipv4_network
    subscription.router.router_ias_lt_ipv4_network = router_ias_lt_ipv4_network
    subscription.router.router_ias_lt_ipv6_network = router_ias_lt_ipv6_network

    subscription = router.RouterProvisioning.from_other_lifecycle(subscription, SubscriptionLifecycle.PROVISIONING)

    return {"subscription": subscription}


@workflow(
    "Import router",
    initial_input_form=initial_input_form_generator,
    target=Target.SYSTEM,
)
def import_router() -> StepList:
    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
